@@ -1,28 +1,20 @@
-# -*- coding: utf-8 -*-
 """Downloading utilities for urls"""
 import copy
 import hashlib
 import logging
-import sys
-
 from os import remove
-from os.path import splitext, join, exists, isfile, split
+from os.path import exists, isfile, join, split, splitext
 from pathlib import Path
-from typing import Optional, Dict, Iterator, Union, List, Any, Callable, Tuple
-if sys.version_info[:2] >= (3, 7):
-    OrderedDict = dict
-else:
-    from collections import OrderedDict
+from typing import Any, Callable, Dict, Iterator, List, Optional, Tuple, Union
+from urllib.parse import parse_qsl, urlencode, urlsplit, urlunsplit
 
 import requests
 import tabulator
-from ratelimit import sleep_and_retry, RateLimitDecorator
+from ratelimit import RateLimitDecorator, sleep_and_retry
 from requests import Request
 from ruamel.yaml import YAML
-from six.moves.urllib.parse import urlsplit, urlunsplit, parse_qsl, urlencode
 
-from hdx.utilities import raisefrom
-from hdx.utilities.path import get_temp_dir, get_filename_from_url
+from hdx.utilities.path import get_filename_from_url, get_temp_dir
 from hdx.utilities.session import get_session
 
 logger = logging.getLogger(__name__)
@@ -32,7 +24,7 @@ class DownloadError(Exception):
     pass
 
 
-class Download(object):
+class Download:
     """Download class with various download operations. Requires either global user agent to be set or appropriate
     user agent parameter(s) to be completed.
 
@@ -56,19 +48,35 @@ class Download(object):
         allowed_methods (iterable): HTTP methods for which to force retry. Defaults t0 frozenset(['GET']).
     """
 
-    def __init__(self, user_agent=None, user_agent_config_yaml=None, user_agent_lookup=None, use_env=True,
-                 fail_on_missing_file=True, rate_limit=None, **kwargs):
-        # type: (Optional[str], Optional[str], Optional[str], bool, bool, Optional[Dict], Any) -> None
-        self.session = get_session(user_agent, user_agent_config_yaml, user_agent_lookup, use_env, fail_on_missing_file, **kwargs)
+    def __init__(
+        self,
+        user_agent: Optional[str] = None,
+        user_agent_config_yaml: Optional[str] = None,
+        user_agent_lookup: Optional[str] = None,
+        use_env: bool = True,
+        fail_on_missing_file: bool = True,
+        rate_limit: Optional[Dict] = None,
+        **kwargs: Any,
+    ) -> None:
+        self.session = get_session(
+            user_agent,
+            user_agent_config_yaml,
+            user_agent_lookup,
+            use_env,
+            fail_on_missing_file,
+            **kwargs,
+        )
         self.response = None
         if rate_limit is not None:
             self.setup = sleep_and_retry(
-                RateLimitDecorator(calls=rate_limit['calls'], period=rate_limit['period']).__call__(self.normal_setup))
+                RateLimitDecorator(
+                    calls=rate_limit["calls"], period=rate_limit["period"]
+                ).__call__(self.normal_setup)
+            )
         else:
             self.setup = self.normal_setup
 
-    def close_response(self):
-        # type: () -> None
+    def close_response(self) -> None:
         """Close response
 
         Returns:
@@ -81,8 +89,7 @@ class Download(object):
             except Exception:
                 pass
 
-    def close(self):
-        # type: () -> None
+    def close(self) -> None:
         """Close response and session
 
         Returns:
@@ -92,17 +99,20 @@ class Download(object):
         self.close_response()
         self.session.close()
 
-    def __enter__(self):
-        # type: () -> 'Download'
+    def __enter__(self) -> "Download":
         return self
 
-    def __exit__(self, exc_type, exc_value, traceback):
-        # type: (Any, Any, Any) -> None
+    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None:
         self.close()
 
     @staticmethod
-    def get_path_for_url(url, folder=None, filename=None, path=None, overwrite=False):
-        # type: (str, Optional[str], Optional[str], Optional[str], bool) -> str
+    def get_path_for_url(
+        url: str,
+        folder: Optional[str] = None,
+        filename: Optional[str] = None,
+        path: Optional[str] = None,
+        overwrite: bool = False,
+    ) -> str:
         """Get filename from url and join to provided folder or temporary folder if no folder supplied, ensuring uniqueness
 
         Args:
@@ -118,14 +128,16 @@ class Download(object):
         """
         if path:
             if folder or filename:
-                raise DownloadError('Cannot use folder or filename and path arguments together!')
+                raise DownloadError(
+                    "Cannot use folder or filename and path arguments together!"
+                )
             folder, filename = split(path)
         if not filename:
             filename = get_filename_from_url(url)
         filename, extension = splitext(filename)
         if not folder:
             folder = get_temp_dir()
-        path = join(folder, '%s%s' % (filename, extension))
+        path = join(folder, f"{filename}{extension}")
         if overwrite:
             try:
                 remove(path)
@@ -135,11 +147,10 @@ class Download(object):
             count = 0
             while exists(path):
                 count += 1
-                path = join(folder, '%s%d%s' % (filename, count, extension))
+                path = join(folder, f"{filename}{count}{extension}")
         return path
 
-    def get_full_url(self, url):
-        # type: (str) -> str
+    def get_full_url(self, url: str) -> str:
         """Get full url including any additional parameters
 
         Args:
@@ -148,13 +159,12 @@ class Download(object):
         Returns:
             str: Full url including any additional parameters
         """
-        request = Request('GET', url)
+        request = Request("GET", url)
         preparedrequest = self.session.prepare_request(request)
         return preparedrequest.url
 
     @staticmethod
-    def get_url_for_get(url, parameters=None):
-        # type: (str, Optional[Dict]) -> str
+    def get_url_for_get(url: str, parameters: Optional[Dict] = None) -> str:
         """Get full url for GET request including parameters
 
         Args:
@@ -166,15 +176,16 @@ class Download(object):
 
         """
         spliturl = urlsplit(url)
-        getparams = OrderedDict(parse_qsl(spliturl.query))
+        getparams = dict(parse_qsl(spliturl.query))
         if parameters is not None:
             getparams.update(parameters)
         spliturl = spliturl._replace(query=urlencode(getparams))
         return urlunsplit(spliturl)
 
     @staticmethod
-    def get_url_params_for_post(url, parameters=None):
-        # type: (str, Optional[Dict]) -> Tuple[str, Dict]
+    def get_url_params_for_post(
+        url: str, parameters: Optional[Dict] = None
+    ) -> Tuple[str, Dict]:
         """Get full url for POST request and all parameters including any in the url
 
         Args:
@@ -186,16 +197,17 @@ class Download(object):
 
         """
         spliturl = urlsplit(url)
-        getparams = OrderedDict(parse_qsl(spliturl.query))
+        getparams = dict(parse_qsl(spliturl.query))
         if parameters is not None:
             getparams.update(parameters)
-        spliturl = spliturl._replace(query='')
+        spliturl = spliturl._replace(query="")
         full_url = urlunsplit(spliturl)
         return full_url, getparams
 
     @staticmethod
-    def hxl_row(headers, hxltags, dict_form=False):
-        # type: (List[str], Dict[str,str], bool) -> Union[List[str], Dict[str,str]]
+    def hxl_row(
+        headers: List[str], hxltags: Dict[str, str], dict_form: bool = False
+    ) -> Union[List[str], Dict[str, str]]:
         """Return HXL tag row for header row given list of headers and dictionary with header to HXL hashtag mappings.
         Return list or dictionary depending upon the dict_form argument.
 
@@ -209,12 +221,20 @@ class Download(object):
 
         """
         if dict_form:
-            return {header: hxltags.get(header, '') for header in headers}
+            return {header: hxltags.get(header, "") for header in headers}
         else:
-            return [hxltags.get(header, '') for header in headers]
+            return [hxltags.get(header, "") for header in headers]
 
-    def normal_setup(self, url, stream=True, post=False, parameters=None, timeout=None, headers=None, encoding=None):
-        # type: (str, bool, bool, Optional[Dict], Optional[float], Optional[Dict], Optional[str]) -> requests.Response
+    def normal_setup(
+        self,
+        url: str,
+        stream: bool = True,
+        post: bool = False,
+        parameters: Optional[Dict] = None,
+        timeout: Optional[float] = None,
+        headers: Optional[Dict] = None,
+        encoding: Optional[str] = None,
+    ) -> requests.Response:
         """Setup download from provided url returning the response
 
         Args:
@@ -238,22 +258,32 @@ class Download(object):
                 if isfile(url):
                     url = Path(url).resolve().as_uri()
                 else:
-                    spliturl = spliturl._replace(scheme='http')
+                    spliturl = spliturl._replace(scheme="http")
                     url = urlunsplit(spliturl)
             if post:
                 full_url, parameters = self.get_url_params_for_post(url, parameters)
-                self.response = self.session.post(full_url, data=parameters, stream=stream, timeout=timeout, headers=headers)
+                self.response = self.session.post(
+                    full_url,
+                    data=parameters,
+                    stream=stream,
+                    timeout=timeout,
+                    headers=headers,
+                )
             else:
-                self.response = self.session.get(self.get_url_for_get(url, parameters), stream=stream, timeout=timeout, headers=headers)
+                self.response = self.session.get(
+                    self.get_url_for_get(url, parameters),
+                    stream=stream,
+                    timeout=timeout,
+                    headers=headers,
+                )
             self.response.raise_for_status()
             if encoding:
                 self.response.encoding = encoding
         except Exception as e:
-            raisefrom(DownloadError, 'Setup of Streaming Download of %s failed!' % url, e)
+            raise DownloadError(f"Setup of Streaming Download of {url} failed!") from e
         return self.response
 
-    def hash_stream(self, url):
-        # type: (str) -> str
+    def hash_stream(self, url: str) -> str:
         """Stream file from url and hash it using MD5. Must call setup method first.
 
         Args:
@@ -269,11 +299,19 @@ class Download(object):
                 if chunk:  # filter out keep-alive new chunks
                     md5hash.update(chunk)
             return md5hash.hexdigest()
-        except Exception as e:
-            raisefrom(DownloadError, 'Download of %s failed in retrieval of stream!' % url, e)
+        except Exception:
+            raise DownloadError(
+                f"Download of {url} failed in retrieval of stream!" % url
+            )
 
-    def stream_file(self, url, folder=None, filename=None, path=None, overwrite=False):
-        # type: (str, Optional[str], Optional[str], Optional[str], bool) -> str
+    def stream_file(
+        self,
+        url: str,
+        folder: Optional[str] = None,
+        filename: Optional[str] = None,
+        path: Optional[str] = None,
+        overwrite: bool = False,
+    ) -> str:
         """Stream file from url and store in provided folder or temporary folder if no folder supplied.
         Must call setup method first.
 
@@ -291,21 +329,33 @@ class Download(object):
         path = self.get_path_for_url(url, folder, filename, path, overwrite)
         f = None
         try:
-            f = open(path, 'wb')
+            f = open(path, "wb")
             for chunk in self.response.iter_content(chunk_size=10240):
                 if chunk:  # filter out keep-alive new chunks
                     f.write(chunk)
                     f.flush()
             return f.name
         except Exception as e:
-            raisefrom(DownloadError, 'Download of %s failed in retrieval of stream!' % url, e)
+            raise DownloadError(
+                f"Download of {url} failed in retrieval of stream!"
+            ) from e
         finally:
             if f:
                 f.close()
 
-    def download_file(self, url, folder=None, filename=None, path=None, overwrite=False,
-                      post=False, parameters=None, timeout=None, headers=None, encoding=None):
-        # type: (str, Optional[str], Optional[str], Optional[str], bool, bool, Optional[Dict], Optional[float], Optional[Dict], Optional[str]) -> str
+    def download_file(
+        self,
+        url: str,
+        folder: Optional[str] = None,
+        filename: Optional[str] = None,
+        path: Optional[str] = None,
+        overwrite: bool = False,
+        post: bool = False,
+        parameters: Optional[Dict] = None,
+        timeout: Optional[float] = None,
+        headers: Optional[Dict] = None,
+        encoding: Optional[str] = None,
+    ) -> str:
         """Download file from url and store in provided folder or temporary folder if no folder supplied
 
         Args:
@@ -324,11 +374,26 @@ class Download(object):
             str: Path of downloaded file
 
         """
-        self.setup(url, stream=True, post=post, parameters=parameters, timeout=timeout, headers=headers, encoding=encoding)
+        self.setup(
+            url,
+            stream=True,
+            post=post,
+            parameters=parameters,
+            timeout=timeout,
+            headers=headers,
+            encoding=encoding,
+        )
         return self.stream_file(url, folder, filename, path, overwrite)
 
-    def download(self, url, post=False, parameters=None, timeout=None, headers=None, encoding=None):
-        # type: (str, bool, Optional[Dict], Optional[float], Optional[Dict], Optional[str]) -> requests.Response
+    def download(
+        self,
+        url: str,
+        post: bool = False,
+        parameters: Optional[Dict] = None,
+        timeout: Optional[float] = None,
+        headers: Optional[Dict] = None,
+        encoding: Optional[str] = None,
+    ) -> requests.Response:
         """Download url
 
         Args:
@@ -343,10 +408,17 @@ class Download(object):
             requests.Response: Response
 
         """
-        return self.setup(url, stream=False, post=post, parameters=parameters, timeout=timeout, headers=headers, encoding=encoding)
+        return self.setup(
+            url,
+            stream=False,
+            post=post,
+            parameters=parameters,
+            timeout=timeout,
+            headers=headers,
+            encoding=encoding,
+        )
 
-    def get_header(self, header):
-        # type: (str) -> Any
+    def get_header(self, header: str) -> Any:
         """Get a particular response header of download
 
         Args:
@@ -358,8 +430,7 @@ class Download(object):
         """
         return self.response.headers.get(header)
 
-    def get_headers(self):
-        # type: () -> Any
+    def get_headers(self) -> Any:
         """Get response headers of download
 
         Returns:
@@ -368,8 +439,7 @@ class Download(object):
         """
         return self.response.headers
 
-    def get_status(self):
-        # type: () -> int
+    def get_status(self) -> int:
         """Get response status code
 
         Returns:
@@ -378,8 +448,7 @@ class Download(object):
         """
         return self.response.status_code
 
-    def get_text(self):
-        # type: () -> str
+    def get_text(self) -> str:
         """Get text content of download
 
         Returns:
@@ -388,8 +457,7 @@ class Download(object):
         """
         return self.response.text
 
-    def get_yaml(self):
-        # type: () -> Any
+    def get_yaml(self) -> Any:
         """Get YAML content of download
 
         Returns:
@@ -399,18 +467,16 @@ class Download(object):
         with YAML() as yaml:
             return yaml.load(self.response.text)
 
-    def get_json(self):
-        # type: () -> Any
+    def get_json(self) -> Any:
         """Get JSON content of download
 
         Returns:
             Any: JSON content of download
 
         """
-        return self.response.json(object_pairs_hook=OrderedDict)
+        return self.response.json()
 
-    def get_tabular_stream(self, url, **kwargs):
-        # type: (str, Any) -> tabulator.Stream
+    def get_tabular_stream(self, url: str, **kwargs: Any) -> tabulator.Stream:
         """Get Tabulator stream.
 
         Args:
@@ -425,21 +491,20 @@ class Download(object):
 
         """
         self.close_response()
-        file_type = kwargs.get('file_type')
+        file_type = kwargs.get("file_type")
         if file_type is not None:
-            kwargs['format'] = file_type
-            del kwargs['file_type']
-        if 'http_session' not in kwargs:
-            kwargs['http_session'] = self.session
+            kwargs["format"] = file_type
+            del kwargs["file_type"]
+        if "http_session" not in kwargs:
+            kwargs["http_session"] = self.session
         try:
             self.response = tabulator.Stream(url, **kwargs)
             self.response.open()
             return self.response
         except Exception as e:
-            raisefrom(DownloadError, 'Getting tabular stream for %s failed!' % url, e)
+            raise DownloadError(f"Getting tabular stream for {url} failed!") from e
 
-    def get_tabular_rows_as_list(self, url, **kwargs):
-        # type: (str, Any) -> Iterator[List]
+    def get_tabular_rows_as_list(self, url: str, **kwargs: Any) -> Iterator[List]:
         """Get iterator for reading rows from tabular data. Each row is returned as a list.
 
         Args:
@@ -455,8 +520,18 @@ class Download(object):
         """
         return self.get_tabular_stream(url, **kwargs).iter(keyed=False)
 
-    def get_tabular_rows(self, url, headers=1, dict_form=False, ignore_blank_rows=True, header_insertions=None, row_function=None, **kwargs):
-        # type: (str, Union[int, List[int], List[str]], bool, bool, Optional[List[Tuple[int,str]]], Optional[Callable[[List[str],Union[List,Dict]],Union[List,Dict]]], Any) -> Tuple[List[str],Iterator[Union[List,Dict]]]
+    def get_tabular_rows(
+        self,
+        url: str,
+        headers: Union[int, List[int], List[str]] = 1,
+        dict_form: bool = False,
+        ignore_blank_rows: bool = True,
+        header_insertions: Optional[List[Tuple[int, str]]] = None,
+        row_function: Optional[
+            Callable[[List[str], Union[List, Dict]], Union[List, Dict]]
+        ] = None,
+        **kwargs: Any,
+    ) -> Tuple[List[str], Iterator[Union[List, Dict]]]:
         """Returns header of tabular file pointed to by url and an iterator where each row is returned as a list
         or dictionary depending on the dict_rows argument. The headers argument is either a row number or list of row
         numbers (in case of multi-line headers) to be considered as headers (rows start counting at 1), or the actual
@@ -485,12 +560,12 @@ class Download(object):
 
         """
         if headers is None:
-            raise DownloadError('Argument headers cannot be None!')
+            raise DownloadError("Argument headers cannot be None!")
         if ignore_blank_rows:
-            skip_rows = kwargs.get('skip_rows', list())
-            if {'type': 'preset', 'value': 'blank'} not in skip_rows:
-                skip_rows.append({'type': 'preset', 'value': 'blank'})
-            kwargs['skip_rows'] = skip_rows
+            skip_rows = kwargs.get("skip_rows", list())
+            if {"type": "preset", "value": "blank"} not in skip_rows:
+                skip_rows.append({"type": "preset", "value": "blank"})
+            kwargs["skip_rows"] = skip_rows
         stream = self.get_tabular_stream(url, headers=headers, **kwargs)
         origheaders = stream.headers
         if header_insertions is None or origheaders is None:
@@ -513,8 +588,7 @@ class Download(object):
 
         return headers, get_next()
 
-    def download_tabular_key_value(self, url, **kwargs):
-        # type: (str, Any) -> Dict
+    def download_tabular_key_value(self, url: str, **kwargs: Any) -> Dict:
         """Download 2 column csv from url and return a dictionary of keys (first column) and values (second column)
 
         Args:
@@ -535,8 +609,13 @@ class Download(object):
             output_dict[row[0]] = row[1]
         return output_dict
 
-    def download_tabular_rows_as_dicts(self, url, headers=1, keycolumn=1, **kwargs):
-        # type: (str, Union[int, List[int], List[str]], int, Any) -> Dict[Dict]
+    def download_tabular_rows_as_dicts(
+        self,
+        url: str,
+        headers: Union[int, List[int], List[str]] = 1,
+        keycolumn: int = 1,
+        **kwargs: Any,
+    ) -> Dict[str, Dict]:
         """Download multicolumn csv from url and return dictionary where keys are first column and values are
         dictionaries with keys from column headers and values from columns beneath
 
@@ -549,11 +628,11 @@ class Download(object):
             delimiter (Optional[str]): Delimiter used for values in each row. Defaults to inferring.
 
         Returns:
-            Dict[Dict]: Dictionary where keys are first column and values are dictionaries with keys from column
+            Dict[str,Dict]: Dictionary where keys are first column and values are dictionaries with keys from column
             headers and values from columns beneath
 
         """
-        kwargs['headers'] = headers
+        kwargs["headers"] = headers
         stream = self.get_tabular_stream(url, **kwargs)
         output_dict = dict()
         headers = stream.headers
@@ -568,8 +647,13 @@ class Download(object):
                     output_dict[first_val][header] = row[header]
         return output_dict
 
-    def download_tabular_cols_as_dicts(self, url, headers=1, keycolumn=1, **kwargs):
-        # type: (str, Union[int, List[int], List[str]], int, Any) -> Dict[Dict]
+    def download_tabular_cols_as_dicts(
+        self,
+        url: str,
+        headers: Union[int, List[int], List[str]] = 1,
+        keycolumn: int = 1,
+        **kwargs: Any,
+    ) -> Dict[str, Dict]:
         """Download multicolumn csv from url and return dictionary where keys are header names and values are
         dictionaries with keys from first column and values from other columns
 
@@ -582,11 +666,11 @@ class Download(object):
             delimiter (Optional[str]): Delimiter used for values in each row. Defaults to inferring.
 
         Returns:
-            Dict[Dict]: Dictionary where keys are header names and values are dictionaries with keys from first column
+            Dict[str,Dict]: Dictionary where keys are header names and values are dictionaries with keys from first column
             and values from other columns
 
         """
-        kwargs['headers'] = headers
+        kwargs["headers"] = headers
         stream = self.get_tabular_stream(url, **kwargs)
         output_dict = dict()
         headers = stream.headers
@@ -603,8 +687,7 @@ class Download(object):
         return output_dict
 
     @staticmethod
-    def get_column_positions(headers):
-        # type: (List[str]) -> Dict[str,int]
+    def get_column_positions(headers: List[str]) -> Dict[str, int]:
         """Get mapping of headers to column positions
 
         Args:
